package gov.va.med.attachment.spring.dao;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.List;

import javax.sql.DataSource;

import org.apache.log4j.Logger;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.ResultSetExtractor;
import org.springframework.jdbc.core.RowMapper;

import gov.va.med.attachment.exceptions.ArsDataAccessException;
import gov.va.med.attachment.spring.model.ClaimAttachments;
import gov.va.med.attachment.spring.model.ClaimDetails;
import gov.va.med.attachment.spring.model.UserAuthorization;
import gov.va.med.attachment.util.DateUtils;
import gov.va.med.attachment.util.FileUtils;

public class ClaimDetailsDAOImpl extends BaseDao implements ClaimDetailsDAO{
	
	
	private JdbcTemplate jdbcTemplate;
	private JdbcTemplate msSqlTemplate;
	
	public ClaimDetailsDAOImpl(DataSource dataSource, DataSource msSqlDataSource ) {
		jdbcTemplate = new JdbcTemplate(dataSource);
		msSqlTemplate = new JdbcTemplate(msSqlDataSource);
	}
	
	static final Logger auditLogger = Logger.getLogger("auditLogger");
	static final Logger errorLogger = Logger.getLogger("errorLogger");

	
	@Override
	public UserAuthorization getUserAuth(String userId)
	{
		String sql = "SELECT * FROM [Claim_Attachments].[dbo].UserAuthorization WHERE UserId= '" + userId + "'";
		return msSqlTemplate.query(sql, new ResultSetExtractor<UserAuthorization>() {

			@Override
			public UserAuthorization extractData(ResultSet rs) throws SQLException,
					DataAccessException {
				if (rs.next()) {
					UserAuthorization userAuthorization = new UserAuthorization();
					userAuthorization.setUserId(rs.getString("UserId"));
					userAuthorization.setDomain(rs.getString("DOMAIN"));
					userAuthorization.setStatus(rs.getString("STATUS"));
					userAuthorization.setDownload(rs.getString("DOWNLOAD"));
					userAuthorization.setEdit(rs.getString("EDIT"));
					userAuthorization.setDelete(rs.getString("DEL"));
					userAuthorization.setAdmin(rs.getString("ADMIN"));

					return userAuthorization;
				}
				
				return null;
			}
			
		});
	}
	
	
	@Override
	public void search(ClaimDetails claimDetails){
		
		String sql = "select  c.patient_control_number, c.bill_type_cd, c.medical_rec_nbr, c.begin_service_date, c.end_service_date, "
				+ "pr.last_name, pr.provider_tin, pr.provider_npi, "
				+ "p.first_name, p.last_name, p.person_id "
				+ "FROM fpps_owner.claims c "
				+ "INNER JOIN fpps_owner.provider_info pr ON c.claim_index = pr.claim_index "
				+ "INNER JOIN fpps_owner.person_info p ON c.claim_index = p.claim_index "
				+ "WHERE c.claim_index = " + claimDetails.getClaimId();
		
		Connection conn;
		ResultSet rs;
		String id = "";
		try {
			
			conn = jdbcTemplate.getDataSource().getConnection();
			rs = getResultSet(conn, sql);
			if (rs.next()) //if there is a record
			{
				id       = rs.getString(1);
				
				claimDetails.setPatientControlNumber(rs.getString(1));
				claimDetails.setBillTypeCd(rs.getString(2));
				claimDetails.setMedicalRecordNumber(rs.getString(3));
				claimDetails.setClaimServiceStartDate(new SimpleDateFormat("MM/dd/yyyy").format((java.sql.Timestamp) rs.getTimestamp(4)) );
				claimDetails.setClaimServiceEndDate(new SimpleDateFormat("MM/dd/yyyy").format((java.sql.Timestamp) rs.getTimestamp(5)) );
				claimDetails.setProviderName(rs.getString(6));
				claimDetails.setProviderTin(rs.getString(7));
				claimDetails.setProviderNPI(rs.getString(8));
				claimDetails.setPatientFirstName(rs.getString(9));
				claimDetails.setPatientLastName(rs.getString(10));
				claimDetails.setPatientSsn(rs.getString(11));
			}
			
		} catch (Exception e) {
			String message = "Unable to download attachment : " + e.getMessage();
			errorLogger.error("Exception thrown when downloading attachment : " + e.getMessage() );
			throw new ArsDataAccessException(message ,e);
		} finally {
			
		}
		
		
	}
	
	@Override
	public void save(ClaimDetails claimDetails){
		
		
		Connection sqlConn;
		try {
			sqlConn = msSqlTemplate.getDataSource().getConnection();

/*			String sql = "INSERT INTO [PD_EDI].[Dflt].ClaimDetails (CLAIM_INDEX, PATIENT_CONTROL_NUMBER, BILL_TYPE_CD, "
					+ "MEDICAL_REC_NBR, BEGIN_SERVICE_DATE, END_SERVICE_DATE, PROVIDER_NAME, PROVIDER_TIN, PROVIDER_NPI, "
					+ "PATIENT_FIRST_NAME, PATIENT_LAST_NAME, PERSON_ID ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";*/
			
			
			
			String sql = "INSERT INTO [dbo].ClaimDetails (CLAIM_INDEX, PATIENT_CONTROL_NUMBER, BILL_TYPE_CD, "
					+ "MEDICAL_REC_NBR, BEGIN_SERVICE_DATE, END_SERVICE_DATE, PROVIDER_NAME, PROVIDER_TIN, PROVIDER_NPI, "
					+ "PATIENT_FIRST_NAME, PATIENT_LAST_NAME, PERSON_ID ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
			
			msSqlTemplate.update(sql, claimDetails.getClaimId(), claimDetails.getPatientControlNumber(), claimDetails.getBillTypeCd(), 
					claimDetails.getMedicalRecordNumber(), claimDetails.getClaimServiceStartDate(), claimDetails.getClaimServiceEndDate(), 
					claimDetails.getProviderName(), claimDetails.getProviderTin(), claimDetails.getProviderNPI(), claimDetails.getPatientFirstName(), 
					claimDetails.getPatientLastName(), claimDetails.getPatientSsn());
			
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	@Override
	public List<ClaimDetails> list() {
		
			String sql = "SELECT * FROM [dbo].ClaimDetails";
			List<ClaimDetails> listclaimDetails = msSqlTemplate.query(sql, new RowMapper<ClaimDetails>() {
	
				@Override
				public ClaimDetails mapRow(ResultSet rs, int rowNum) throws SQLException {
					try {
						ClaimDetails claimDetails = new ClaimDetails();
	
						claimDetails.setClaimId(rs.getInt("CLAIM_INDEX"));
						claimDetails.setPatientControlNumber(rs.getString("PATIENT_CONTROL_NUMBER"));
						claimDetails.setBillTypeCd(rs.getString("BILL_TYPE_CD"));
						claimDetails.setMedicalRecordNumber(rs.getString("MEDICAL_REC_NBR"));
						claimDetails.setClaimServiceStartDate(rs.getString("BEGIN_SERVICE_DATE"));
						claimDetails.setClaimServiceEndDate(rs.getString("END_SERVICE_DATE"));
						claimDetails.setProviderName(rs.getString("PROVIDER_NAME"));
						claimDetails.setProviderTin(rs.getString("PROVIDER_TIN"));
						claimDetails.setProviderNPI(rs.getString("PROVIDER_NPI"));
						claimDetails.setPatientFirstName(rs.getString("PATIENT_FIRST_NAME"));
						claimDetails.setPatientLastName(rs.getString("PATIENT_LAST_NAME"));
						claimDetails.setPatientSsn(rs.getString("PERSON_ID"));
	
						try {
							claimDetails.setClaimServiceStartDate(DateUtils.toEnglishDate(rs.getDate("BEGIN_SERVICE_DATE")));
							claimDetails.setClaimServiceEndDate(DateUtils.toEnglishDate(rs.getDate("END_SERVICE_DATE")));
						} catch (Exception e) {
							String message = "Unable to list attachments : " + e.getMessage();
							throw new ArsDataAccessException(message ,e);
						}
									
						return claimDetails;
					} catch (Exception e) {
						String message = "Unable to list claimDetails : " + e.getMessage();
						errorLogger.error("Exception thrown when listing claim details : " + e.getMessage() );
						throw new ArsDataAccessException(message ,e);
					}
				}
				
			});
		
		return listclaimDetails;
	}
	
	
}
